/* SPDX-License-Identifier: LGPL-2.1+ */
/******************************************************************************
 * Copyright (c) Huawei Technologies Co., Ltd. 2020. Allrights reserved
 * Description: isulad utils
 * Author: lifeng
 * Create: 2020-04-11
******************************************************************************/
#ifndef __ISULAD_UTILS_H
#define __ISULAD_UTILS_H

#include <stdio.h>
#include <stdbool.h>
#include <pwd.h>

#include "compiler.h"

/* isulad: replace space with SPACE_MAGIC_STR */
#define SPACE_MAGIC_STR "[#)"

/* isulad:
 ld cutime, cstime, priority, nice, timeout, it_real_value, rss,
 c  state,
 d  ppid, pgrp, session, tty, tpgid,
 s  signal, blocked, sigignore, sigcatch,
 lu flags, min_flt, cmin_flt, maj_flt, cmaj_flt, utime, stime,
 lu rss_rlim, start_code, end_code, start_stack, kstk_esp, kstk_eip,
 lu start_time, vsize, wchan, nswap, cnswap,
*/

/* Basic data structure which holds all information we can get about a process.
 * (unless otherwise specified, fields are read from /proc/#/stat)
 *
 * Most of it comes from task_struct in linux/sched.h
 */
typedef struct proc_t {
    // 1st 16 bytes
    int pid;        /* process id */
    int ppid;       /* pid of parent process */

    char state;     /* single-char code for process state (S=sleeping) */

    unsigned long long
    utime,      /* user-mode CPU time accumulated by process */
    stime,      /* kernel-mode CPU time accumulated by process */
    // and so on...
    cutime,     /* cumulative utime of process and reaped children */
    cstime,     /* cumulative stime of process and reaped children */
    start_time; /* start time of process -- seconds since 1-1-70 */

    long
    priority,   /* kernel scheduling priority */
    timeout,    /* ? */
    nice,       /* standard unix nice level of process */
    rss,        /* resident set size from /proc/#/stat (pages) */
    it_real_value;  /* ? */
    unsigned long
    rtprio,     /* real-time priority */
    sched,      /* scheduling class */
    vsize,      /* number of pages of virtual memory ... */
    rss_rlim,   /* resident set size limit? */
    flags,      /* kernel flags for the process */
    min_flt,    /* number of minor page faults since process start */
    maj_flt,    /* number of major page faults since process start */
    cmin_flt,   /* cumulative min_flt of process and child processes */
    cmaj_flt,   /* cumulative maj_flt of process and child processes */
    nswap,      /* ? */
    cnswap,     /* cumulative nswap ? */
    start_code, /* address of beginning of code segment */
    end_code,   /* address of end of code segment */
    start_stack,    /* address of the bottom of stack for the process */
    kstk_esp,   /* kernel stack pointer */
    kstk_eip,   /* kernel instruction pointer */
    wchan;      /* address of kernel wait channel proc is sleeping in */

    char cmd[16];   /* basename of executable file in call to exec(2) */
    int
    pgrp,       /* process group id */
    session,    /* session id */
    tty,        /* full device number of controlling terminal */
    tpgid,      /* terminal process group id */
    exit_signal,    /* might not be SIGCHLD */
    processor;      /* current (or most recent?) CPU */
} proc_t;

__hidden extern int lxc_mem_realloc(void **newptr, size_t newsize, void *oldptr, size_t oldsize);
__hidden extern void *lxc_common_calloc_s(size_t size);
__hidden extern char *safe_strdup(const char *src);

__hidden extern int lxc_open(const char *filename, int flags, mode_t mode);
__hidden extern FILE *lxc_fopen(const char *filename, const char *mode);

__hidden extern void lxc_write_error_message(int errfd, const char *format, ...);
__hidden extern int lxc_file2str(const char *filename, char ret[], int cap);
__hidden extern int unsigned long long lxc_get_process_startat(pid_t pid);
// set env home in container
__hidden extern int lxc_setup_env_home(uid_t uid);

__hidden extern bool lxc_process_alive(pid_t pid, unsigned long long start_time);

__hidden extern bool is_non_negative_num(const char *s);

__hidden extern int util_getpwent_r(FILE *stream, struct passwd *resbuf, char *buffer, size_t buflen, struct passwd **result);

__hidden extern ssize_t lxc_write_nointr_for_fifo(int fd, const char *buf, size_t count);

#endif
